# Help.pm - The Object Class that provides a Help Object
# Created by James A. Pattie, 11/07/2000.

package Portal::Accounting::Data::Help;
use strict;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.02';

# Help Data structure
my %Topics = (
             en => {
                     Normal => {
                               "_order_" => "Close,_seperator_,Refresh,Help,_seperator_,Main,Config",
                               "Close" => "This closes the windows related to XIWA.",
                               "Refresh" => "This redisplays the Main Menu of XIWA.",
                               "Help" => "This is what you are currently seeing.",
                               "Main" => {
                                         "_order_" => "Open,Configuration",
                                         "Open" => "Open an existing Ledger.",
                                         "Configuration" => "Edit XIWA Configuration settings such as Accounts, Sub-Accounts, Ledgers, etc.",
                                      },
                               "Config" => {
                                         "_order_" => "Accounts,SubAccounts,Stocks,Ledgers,SubEntities,Employees,Assets",
					 "Accounts" => "Accounts help goes here",
					 "Ledgers" => {
					   "_order_" => "Select,New,Edit,Delete",
					   "Select" => "Select the Ledger to work with from the combo box.",
                                           "New" => "Create a new Ledger.",
					   "Edit" => "Edit the currently Selected ledger.",
					   "Delete" => "Delete the currently Selected ledger.",
					 },
					 "SubAccounts" => "SubAccounts help goes here",
					 "Stocks" => "Stocks help goes here",
					 "SubEntities" => "Sub-Entity help goes here",
					 "Employees" => "Employees help goes here",
					 "Assets" => "Asset help goes here",
                                      },
                               },
                     System => {
                               "_order_" => "This should never be reached",
                               "This should never be reached" => "The XIWA always runs in Normal mode!<br><br>Go <help app=\"Accounting\" content=\"Normal,Help\">here</help> to return to the Normal Help System.",
                               },
                   },
             );

# new
# instantiates an instance of a Help
# takes: lang
sub new
{
  my $that = shift;
  my $class = ref($that) || $that;
  my $self = bless {}, $class;
  my %args = ( lang => "en", @_ );
  my $errStr = "Portal::Accounting::Data::Help->new() - Error!<br>\n";

  $self->{lang} = $args{lang};  # we will assume this language is valid as the Language module will have already validated it for us.
  $self->{Topics} = \%Topics;   # store the help data in the class.
  if (!exists $self->{Topics}->{$self->{lang}})
  {
    $self->{lang} = "en";  # This must always exist!
  }

  # do validation
  $self->{error} = !$self->isValid;
  if ($self->{error})
  {
    $self->{errorString} = $errStr . $self->{errorString};
  }

  return $self;
}

# isValid - Returns 0 or 1 to indicate if the object is valid.
sub isValid
{
  my $self = shift;
  my $error = 0;
  my $errorString = "";
  my $errStr = "Portal::Accounting::Data::Help->isValid() - Error!<br>\n";

  # do validation code here.

  if (length $errorString > 0)
  {
    $error = 1;
    $self->{error} = $error;
    $self->{errorString} = $errStr;
    $self->{errorString} .= $errorString;
  }

  return !$error;
}

sub DESTROY
{
  my $self = shift;
}

sub AUTOLOAD
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  if (@_)
  {
    return $self->{$name} = shift;
  }
  else
  {
    return $self->{$name};
  }
}

# setError
# requires: errorString
# returns: nothing
sub setError
{
  my $self = shift;
  my %args = ( errorString => "", @_ );
  my $errorString = $args{errorString};

  $self->{errorString} = $errorString;
  $self->{error} = 1;
}

# didErrorOccur
# returns the value of error.
sub didErrorOccur
{
  my $self = shift;

  return $self->{error};
}

# errorMessage
# returns the value of errorString
sub errorMessage
{
  my $self = shift;

  return $self->{errorString};
}

1;
__END__

=head1 NAME

Help - Object used to build a Help Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Data::Help;
  my $obj = Portal::Accounting::Data::Help->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

Help is a Help class.

=head1 Exported FUNCTIONS

  scalar new(lang)
    Creates a new instance of the Portal::Accounting::Data::Help
    object.

  bool isValid(void)
    Returns 0 or 1 to indicate if the object is valid.  The error will be
    available via errorMessage().
    
  void setError(errorString)
    Sets error = 1 and errorString = errorString.

  scalar didErrorOccur(void)
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.

=head1 Exported FUNCTIONS

  Topics - Help topics hash.  The language code is the first index,
           followed by Normal and System.  They provide the main indexes
           for the Normal and System modes.  Under them are the topics
           of interest for the 2 modes.

  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Help(3)

=cut
