# Buy.pm - The Object Class that provides a Buy Object
#$Id: Buy.pm,v 1.22.4.3 2006/08/07 13:41:59 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2005 http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Investments::Objects::Buy;
use strict;
use Portal::Accounting::Investments::Objects::Event;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT %Elements);

require Exporter;

@ISA = qw(Portal::Accounting::Investments::Objects::Event Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.22.4.3 $';

=head1 NAME

Buy - Object used to create, edit?, save a Buy event.

=head1 SYNOPSIS

  use Portal::Accounting::Investments::Objects::Buy;
  my $obj = Portal::Accounting::Investments::Objects::Buy->new();
  $obj->populate(date =>, . . .  );
  if ($obj->error)
  {
    die $errorMessage();
  }

=head1 DESCRIPTION

  Buy is the class used to manage buy security events.  It contains forms
  for creating, editing (if this gets implemented), and deleting buy events.
  It also contains the database functionality for inserting and deleting
  records.  As with all security events LotNode is used as part of the
  processes to create, delete, and insert data.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(dbHandle => , langObj =>, ledgerCode =>)

  Creates a new instance of the Portal::Accounting::Investments::Objects::Buy module.

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( ledgerCode => "", app => "Accounting", state => "Investments::Journal", input => undef, @_ );

  # instantiate anything unique to this module
  $self->{dbHandle} = $args{dbHandle};
  my ($date, $account, $stock);
  if (defined $args{input})
  {
    $date = $args{input}->{date};
    $account = $args{input}->{account};
    $stock = $args{input}->{stock};
  }
  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }
  # instantiate anything unique to this module
  my $me = "buy";
  my $l = $self->{langObj};
  my @date = $self->getDateArray;
  my $today = $self->getCurrentDate;
  if ($date eq '') { $date = $today; }

  $self->{defaultForm} = "main";
  my $Shares = $l->map("Shares");
  my $Price = $l->map("Price");
  my $Total = $l->map("Total");
  my $Comm = $l->map("Commission");
  $self->{data}->{main} = {
    "app" => { -Type => "hidden", -Value => $args{app} },
    "state" => { -Type => "hidden", -Value => $args{state} },
    "command" => { -Type => "hidden", -Value => "save" },
    "id" => { -Type => "hidden", },
    "counter" => { -Type => "hidden", },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, -DataType => "string" },
    "lot" => { -Type => "hidden", },
    "type" => { -Type => "hidden", -Label => $l->map($me), -Value => $me, -DataType => "string" },
    "buyDate" => { -Label => $l->map("Date"), -Type => "date-picker", -DataType => "string",
      -year => $date[0], -Value => $date },
    "date" => { -Type => "hidden", },
    "account" => { -Label => $l->map("Account"), -Type => "select", -DataType => "int",
      -Options => $self->{accounts},
      -Value => $account,
    },
    "stock" => { -Label => $l->map("Stock"), -Type => "select", -DataType => "int",
      -Options => $self->{stocks},
      -Value => $stock,
    },
    "subEntity" => { -Label => $l->map("Sub Entity"), -Type => "select", -DataType => "int",
      -Options => $self->{subEntities}, -onload => "this.selectedIndex = -1;"      },
    "comment" => { -Type => "text", -DataType => "string", -Label => $l->map("Comment") },
    "shares" => { -Type => "text", -DataType => "float", -Label => $Shares, size => 8, onchange => "setCommission(this); totalCalc(this);"},
    "price" => { -Type => "text", -DataType => "float", -Label => $Price, size => 8, onchange => "totalCalc(this);" },
    "commission" => { -Type => "text", -DataType => "float", -Label => $Comm, size => 8, onchange => "setCommShares; totalCalc(this);" },
    "total" => { -Type => "text", -DataType => "float", -Label => $Total, size => 8, onchange => "totalCalc(this);", },
    "commPerShare" => { -Type => "text", -DataType => "float", -Label => $l->map("Commission")." ".$l->map("Per Share"), size => 8, onchange => "setCommission(this);" },
  };
  $self->{data}->{report} = {
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, },
    "id" => { -Type => "hidden", },
    "lot" => { -Type => "text", -ReadOnly => 1, },
    "type" => { -Type => "text", -Label => $l->map($me), -Value => $me, -DataType => "string", -ReadOnly => 1, },
    "date" => { -Label => $l->map("Date"), -Type => "date-picker", -DataType => "string",
      -year => $date[0], -Value => $today, -ReadOnly => 1, },
    "buyDate" => { -Label => $l->map("Date"), -Type => "date-picker", -DataType => "string",
      -year => $date[0], -Value => $today, -ReadOnly => 1, },
    "account" => { -Label => $l->map("Account"), -Type => "select", -DataType => "int",
      -Options => $self->{accounts} , -ReadOnly => 1,  },
    "stock" => { -Label => $l->map("Stock"), -Type => "select", -DataType => "int",
      -Options => $self->{stocks}, -ReadOnly => 1,      },
    "subEntity" => { -Label => $l->map("Sub Entity"), -Type => "select", -DataType => "int",
      -Options => $self->{subEntities}, -ReadOnly => 1,},
    "comment" => { -Type => "text", -DataType => "string", -Label => $l->map("Comment"), -ReadOnly => 1, },
    "shares" => { -Type => "text", -DataType => "float", -Label => $Shares, -ReadOnly => 1,},
    "price" => { -Type => "text", -DataType => "float", -Label => $Price, -ReadOnly => 1,},
    "commission" => { -Type => "text", -DataType => "float", -Label => $Comm, -ReadOnly => 1,},
    "total" => { -Type => "text", -DataType => "float", -Label => $Total, -ReadOnly => 1, -Value => 0,},
  };
  $self->{css}->{main} = <<"ENDOF";
ENDOF
  my $Submit = $l->map("Submit");
  $self->{template}->{main} = <<"ENDOF";
<form #FORMARGS#>
  #FORMERRORSTRING#
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=counter#
  #FIELD=ledgerCode#
  <h4>#LABEL=type##FIELD=type#</h4>
  <table class="primary">
    <tr>
      <td>#LABEL=date#</td><td>#FI=date# #FI=buyDate#</td>
      <td>#LABEL=subEntity#</td><td>#FI=subEntity#</td>
    </tr>
    <tr>
      <td>#LABEL=account#</td><td>#FI=account#</td>
      <td>#LABEL=stock#</td><td>#FI=stock#</td>
    </tr>
    <tr>
      <td>#LABEL=shares#</td><td>#FI=shares#</td>
      <td>#LABEL=price#</td><td>#FI=price#</td>
    </tr>
    <tr>
      <td>#LABEL=commPerShare#</td><td>#FI=commPerShare#</td>
      <td>#LABEL=commission#</td><td>#FI=commission#</td>
    </tr>
    <tr>
      <td>#LABEL=total#</td><td>#FI=total#</td>
      <td>#LABEL=comment#</td><td>#FI=comment#</td>
    </tr>
  </table>
  <input type="submit" name="submit" value="$Submit" onclick="this.disabled = true;">
  &nbsp;&nbsp;&nbsp;&nbsp;AutoChange:
    <input type="radio" name="autoChange" value="shares">$Shares
    <input type="radio" name="autoChange" value="price">$Price
    <input type="radio" name="autoChange" value="commission" CHECKED>$Comm
    <input type="radio" name="autoChange" value="total">$Total<br>
</form>
ENDOF
  $self->{template}->{report} = <<"ENDOF";
<table id="i#VALUE=id#" class="report">
  <tr>
    <td class="date">#FIELD=buyDate#</td>
    <td class="event"><a title="#LABEL=type#">#FIELD=type#</a></td>
    <td class="detail">#FIELD=comment#</td>
    <td class="price">#FIELD=price#</td>
    <td class="shares">#FIELD=shares#</td>
    <td class="cost">#FIELD=total#</td>
    <td class="subEntity">#VALUE=subEntity#</td>
    <td class="lot">#FIELD=lot#</td>
$self->{deleteString}
  </tr>
</table>
ENDOF
  $self->{javascript}->{main} = <<"ENDOF";
ENDOF
  $self->{profile}->{main} = {
    required => [ qw( buyDate shares price total stock account ) ],
    optional => [ qw( subEntity commission counter) ],
    constraints => {
         buyDate => qr/^\d{4}-\d{2}-\d{2}$/,  #need this exact format for comparisons later.  js in datepicker helps format properly
         shares => 'RE_num_real',
         price => 'RE_num_real',
         total => 'RE_num_real',
         stock => qr/^\d+$/,
         counter => qr/^\d+$/,
         account => qr/^\d+$/,
         commission => 'RE_num_real',
       },
  };
  $self->{profile}->{report} = {
    required => [ qw( date shares price total stock account ) ],
    optional => [ qw( subEntity commission counter) ],
    constraints => {
         buyDate => qr/^\d{4}-\d{2}-\d{2}$/,
         shares => 'RE_num_real',
         price => 'RE_num_real',
         total => 'RE_num_real',
         stock => qr/^\d+$/,
         counter => qr/^\d+$/,
         account => qr/^\d+$/,
         commission => 'RE_num_real',
       },
  };

  $self->{INSERTorder}[0] = [ qw( event_code  buy_date  trans_date  counter  sub_e_code  a  s  l  os  ns  cc  ledger_code  event_ref  comment ) ] ;  #insert order for db column names

  $self->{db2data}[0] = {
    event_code => 'type',
    buy_date => 'buyDate',
    trans_date => 'date',
    sub_e_code => 'subEntity',
    a => 'account',
    s => 'stock',
    l => 'lot',
    os => 'price',
    ns => 'shares',
    cc => 'commission',
    ledger_code => 'ledgerCode',
    event_ref => 'id',
  };  #map database columns to ->data hash, not specified assumes same name
  $self->{data2db}[0] = {reverse %{$self->{db2data}[0]}};

#   $self->{SELECT}[0] = "SELECT ##COLUMNS## FROM $self->{table} WHERE event_ref = #FIELD=id#";
#  $self->{INSERT}[0] = "INSERT INTO $self->{table} (##COLUMNS##) VALUES (##VALUES##)";
#  $self->{UPDATE}[0] = "UPDATE $self->{table} SET ##UPDATE## WHERE event_ref = #FIELD=id#";
#   $self->{DELETE}[0] = "DELETE FROM $self->{table} WHERE event_ref = #FIELD=id#";

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut
sub isValid
{
  my $self = shift;

  #fix up dates,
  my $d = $self->get('date');
  if ($d =~ /(\d{4})-(\d{1})-(\d{1})/) { $d = "$1-".sprintf("%02d", $2)."-".sprintf("%02d", $3); }
  $self->set('date', $d);

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->error('');
    return 0;
  }

  # validate our parameters.
  my $total = sprintf("%.2f",$self->get('shares') * $self->get('price') + $self->get('commission') );
  if ($self->get('total') != $total)
  {
    $self->invalid('total',$self->get('total'), "P * S + C = ".$total);
  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item boolean insert

  Inserts the record for this event into database and calls other functions to fill in other records. A buy might be affected by many events in the future.  PropagateDelete isn't used with buy because it isn't needed with a new node. There is no propagate routine in buy because it should never be called based on the way the system works.  A buy is only propagated during insert.

=cut
sub insert
{
  my $self = shift;
  if (!$self->isValid) { return undef; }

  my $transobj = Portal::Accounting::Objects::JournalEntry->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->get('ledgerCode'));
  if ($transobj->error) { $self->error($transobj->errorMessage); return undef; }
  my $buy_date = $self->get('buyDate');
  my $trans_date = $buy_date;
  #check dates for historical and closeout cases.
  my $lastClose = $transobj->{ledger}->lastClose;
  $self->set('date', $buy_date);
  if ($lastClose eq '')
  {
    my $historical = $transobj->{ledger}->preOpenBooksDate;
    if ($buy_date <= $historical)
    {
      $trans_date = $historical;
      $self->set('date', $historical);
    }
  }
  elsif ($buy_date <= $lastClose)
  {
    $self->error('<div class="error">Date cannot be before last close out of '.$lastClose.'</div>');
    return undef;
  }
  my $lotNum = $self->nextLotNumber();
  my $eventRef = $self->nextSecurityEventRef();
  my $counter = $self->nextSecurityEventCounter($trans_date);
  $self->set('id', $eventRef);
  $self->set('lot', $lotNum);
  $self->set('counter', $counter);

  # insert the security event into the database
  my $sql = "INSERT INTO sec_event_tb (event_code, buy_date, trans_date, counter, A, S, L, OS, NS, CC, ledger_code, event_ref, comment, sub_e_code) "
    . "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
  my $plug = [
    $self->get('type'),
    $buy_date,
    $trans_date,
    $counter,
    $self->get('account'),
    $self->get('stock'),
    $lotNum,
    $self->get('price'),
    $self->get('shares'),
    $self->get('commission'),
    $self->get('ledgerCode'),
    $eventRef,
    $self->get('comment'),
    $self->get('subEntity'),
  ];
  $self->{dbHandle}->write(sql => $sql, plug => $plug);
  if ($self->dbHandle->error) { $self->error($self->dbHandle->errorMessage); }
  else
  {
    # calculate total price
    my $totalCost = sprintf("%.2f", ($self->get('price') * $self->get('shares')) + $self->get('commission') );
    # build a JournalEntry for the data
    $transobj->set('journal', $self->{Journal});
    $transobj->set('date', $self->get('date') );
    $transobj->addRow(subEntity => $self->get('subEntity'), account => $self->get('account'), subAccount => $self->get('stock'), dorc => "true", amount => "$totalCost", , comment => $self->get('comment'));
    $transobj->addRow(subEntity => $self->get('subEntity'), account => $self->get('account'), dorc => "false", amount => "$totalCost", comment => $self->get('comment') );
    if ($transobj->error())
    {
      $self->error("<p>Failed to create new transaction object!</p>" . $transobj->errorMessage);
      return undef;
    }

    # insert the transaction into the database
    my $refNum = $transobj->insert();
    if ($transobj->error)
    {
      $self->error("<p>Error inserting JournalEntry!</p>" . $transobj->errorMessage);
      return undef;
    }
    # insert a new lot node into the database (node_id will fill itself in from the unique sequence on the table definition)
    my $lotNodeObj = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->{langObj}, dbHandle => $self->dbHandle, ledgerCode => $self->get('ledgerCode'));
    $lotNodeObj->populate(
      ledgerCode => $self->get('ledgerCode'),
      date => $self->get('date'),
      parentId => -1,
      event => $eventRef,
      journalRef => $refNum,
      account => $self->get('account'),
      subAccount => $self->get('stock'),
      lot => $lotNum,
      buyLot => $lotNum,
      cost => sprintf("%.2f", $totalCost),
      shares => sprintf("%.6f", $self->get('shares') ),
      subEntity => $self->get('subEntity'),
    );
    $lotNodeObj->insert();
    if ($lotNodeObj->error) { $self->error($lotNodeObj->errorMessage); return undef; }
    my $lotNodeID = $lotNodeObj->get('id');

    #does not call SUPER::insertEvent because this is a new node and only needs to be propagated.  this might be a bad thing later!!!
    # propagate the newly created lot into the future
$self->log('Buy: propogating '.$lotNodeID);
    if (! $self->propagateLot($lotNodeObj))
    {
      $self->error("<p>Error propagating lot node (" . $lotNodeID . ") into the future!</p>");
      return undef;
    }
  }

  if (!$self->error) { $self->{dbHandle}->commit; }
  else { $self->{dbHandle}->rollback; }
  return $eventRef;
}

=item boolean select

  This routine selects a record from the database and performs any transforms on the data to fill out the object with fields that are not stored in the database but are needed for display and calculations.

=cut
sub select
{
  my $self = shift;
  my $result = $self->SUPER::select(@_);
  #since total field is not stored in sec_event_tb, recalculate it and set here in object
  if (!$self->error)
  {
    $self->set('total', sprintf("%.2f",$self->get('price') * $self->get('shares') + $self->get('commission') ));
  }
  return $result;
}

=item integer delete

  Routine calls all of the neccessary functions to get rid of this event and all nodes affected by it in lot trees.

=cut
sub delete
{
  my $self = shift;
  return $self->SUPER::deleteEvents(@_);
}

=back

=cut

1;
__END__

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Objects::Base(3)

=cut