# SpinOff.pm - The Object Class that provides a SpinOff Object
#$Id: SpinOff.pm,v 1.15.4.3 2006/08/08 17:12:46 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2005 http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Investments::Objects::SpinOff;
use strict;
use Portal::Accounting::Investments::Objects::Event;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Investments::Objects::Event Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.15.4.3 $';

=head1 NAME

SpinOff

=head1 SYNOPSIS

  use Portal::Accounting::Investments::Objects::SpinOff;
  my $obj = Portal::Accounting::Investments::Objects::SpinOff->new();
  $obj->populate(date =>, . . .  );
  if ($obj->error)
  {
    die $errorMessage();
  }

=head1 DESCRIPTION

  SpinOff is the class used to manage SpinOff security events.  It contains forms
  for creating, editing (if this gets implemented), and deleting SpinOff events.
  It also contains the database functionality for inserting and deleting
  records.  As with all security events LotNode is used as part of the
  processes to create, delete, and insert data.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(dbHandle => , langObj =>, ledgerCode =>)

  Creates a new instance of the Portal::Accounting::Investments::Objects::SpinOff module.

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( ledgerCode => "", app => "Accounting", state => "Investments::Journal", @_ );

  # instantiate anything unique to this module
  $self->{dbHandle} = $args{dbHandle};
  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }
  # instantiate anything unique to this module
  my $me = "spof";
  my $l = $self->{langObj};
  my @date = $self->getDateArray;
  my $today = $self->getCurrentDate;

  $self->{defaultForm} = "main";
  $self->{data}->{main} = {
    "app" => { -Type => "hidden", -Value => $args{app} },
    "state" => { -Type => "hidden", -Value => $args{state} },
    "command" => { -Type => "hidden", -Value => "save" },
    "id" => { -Type => "hidden", },
    "global" => { -Type => "hidden", -Value => '1', -DataType => "integer" },
    "counter" => { -Type => "hidden", -Value => "", -DataType => "integer" },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, -DataType => "string" },
    "type" => { -Type => "hidden", -Label => $l->map($me), -Value => $me, -DataType => "string" },
    "date" => { -Label => $l->map("Date"), -Type => "date-picker", -DataType => "string",
      -year => $date[0],  -Value => $today },
    "oldStock" => { -Label => $l->map("of"), -Type => "select", -DataType => "integer",
      -Options => $self->{stocks}      },
    "newStock" => { -Label => $l->map("of"), -Type => "select", -DataType => "integer",
      -Options => $self->{stocks}      },
    "cost" => { -Type => "text", -DataType => "string", -Label => $l->map("Cost"), size => 4 },
    "comment" => { -Type => "text", -DataType => "string", -Label => $l->map("Comment") },
    "oldShares" => { -Type => "text", -DataType => "float", -Label => $l->map("Shares"), size => 8},
    "newShares" => { -Type => "text", -DataType => "float", -Label => $l->map("New")." ".$l->map("Shares"), size => 8},
  };
  $self->{data}->{report} = {
    "id" => { -Type => "hidden", },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, },
    "type" => { -Type => "text", -Label => $l->map($me), -Value => $me, -DataType => "string", -ReadOnly => 1,},
    "date" => { -Label => $l->map("Date"), -Type => "date-picker", -DataType => "string",
      -year => $date[0],  -Value => $today, -ReadOnly => 1, },
    "oldStock" => { -Label => $l->map("of"), -Type => "select", -DataType => "integer",
      -Options => $self->{stocks}, -ReadOnly => 1,      },
    "newStock" => { -Label => $l->map("of"), -Type => "select", -DataType => "integer",
      -Options => $self->{stocks}, -ReadOnly => 1,      },
    "cost" => { -Type => "text", -DataType => "string", -Label => $l->map("Cost"), -ReadOnly => 1, },
    "comment" => { -Type => "text", -DataType => "string", -Label => $l->map("Comment"), -ReadOnly => 1, },
    "oldShares" => { -Type => "text", -DataType => "float", -Label => $l->map("Shares"), -ReadOnly => 1,},
    "newShares" => { -Type => "text", -DataType => "float", -Label => $l->map("New")." ".$l->map("Shares"), -ReadOnly => 1,},
  };
  $self->{css}->{main} = <<"ENDOF";
ENDOF
  my $app = $self->{data}->{main}->{app}->{-Value};
  my $state = $self->{data}->{main}->{state}->{-Value};
  my $lc = $self->{data}->{main}->{ledgerCode}->{-Value};
  $self->{template}->{main} = <<"ENDOF";
<form #FORMARGS#>
  #FORMERRORSTRING#
  #FI=app#
  #FI=state#
  #FI=command#
  #FI=counter#
  #FI=ledgerCode#
  <h4>#LABEL=type##FI=type#</h4>
  <table class="primary">
    <tr>
      <td>
        <p>#LABEL=date# #FI=date#</p>
        <p>#LANG=youWillHaveTotal#
        #FI=newShares# #LABEL=newShares# #LANG=of# #FI=newStock#<br>
        #LANG=for every#
        #FI=oldShares# #LABEL=oldShares# #LANG=of# #FI=oldStock#</p>
        <p>#FI=cost#\% #LANG=ofCostwill#</p>
        <p>#LABEL=comment# #FI=comment# <b><font color="green">%</font></b></p>
      </td>
    </tr>
  </table>
  <input type="submit" name="submit" value="#LANG=Submit#" onclick="this.disabled = true;">
</form>
<p><b><font color="green">%</font></b> Optional</p>
ENDOF
  $self->{template}->{report} = <<"ENDOF";
<table id="i#VALUE=id#" class="report">
  <tr>
    <td class="date">#FIELD=date#</td>
    <td class="event"><a title="#LABEL=type#">#FIELD=type#</a></td>
    <td class="detail">#FIELD=oldStock# #LANG=spins off# #FIELD=newStock# with #FIELD=cost# % #LANG=cost# @ #FIELD=newShares# #LANG=for# #FIELD=oldShares# : #FIELD=comment#</td>
$self->{deleteString}   
  </tr>
</table>
ENDOF
  $self->{javascript}->{main} = <<"ENDOF";
ENDOF
  $self->{profile}->{main} = {
    required => [ qw( date oldShares newShares cost oldStock newStock ) ],
    optional => [ qw( comment counter) ],
    constraints => {
         date => qr/^\d{4}-\d{2}-\d{2}$/,
         oldShares => 'RE_num_real',
         newShares => 'RE_num_real',
         cost => 'RE_num_real',
         oldStock => qr/^\d+$/,
         newStock => qr/^\d+$/,
         counter => qr/^\d+$/,
       },
  };
  $self->{profile}->{report} = {
    required => [ qw( date oldShares newShares cost oldStock newStock ) ],
    optional => [ qw( comment counter) ],
    constraints => {
         date => qr/^\d{4}-\d{2}-\d{2}$/,
         oldShares => 'RE_num_real',
         newShares => 'RE_num_real',
         cost => 'RE_num_real',
         oldStock => qr/^\d+$/,
         newStock => qr/^\d+$/,
         counter => qr/^\d+$/,
       },
  };
  $self->{INSERTorder}[0] = [ qw( event_code trans_date counter a s os ns cc ledger_code event_ref comment ) ] ; #insert order for db column names
  
  $self->{db2data}[0] = {
    event_code => 'type',
    trans_date => 'date',
    s => 'oldStock',
    a => 'newStock',
    cc => 'cost',
    os => 'oldShares',
    ns => 'newShares',
    ledger_code => 'ledgerCode',
    event_ref => 'id',
  };  #map database columns to ->data hash, not specified assumes same name
  
  $self->{data2db}[0] = {reverse %{$self->{db2data}[0]}};

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut
sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
  if ($self->get('oldStock') eq $self->get('newStock'))
  {
    $self->invalid('newStock', $self->langObj->map('oldstock') , $self->langObj->map('Companies must be different'));
  }
  my $cost = $self->get('cost') < 0;
  if ($cost < 0 || $cost > 100) { $self->invalid('cost', $cost, 'Cost must be between 0% and 100%.'); }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item boolean select

  This routine selects a record from the database and performs any transforms on the data to fill out the object with fields that are not stored in the database but are needed for display and calculations.
  
=cut
sub select
{
  my $self = shift;
  my $r = $self->SUPER::select(@_);
  if (!$self->error) { $self->set('cost', $self->get('cost') * 100.00); }
  return $r;
}

=item boolean insert

  Inserts the record for this event into database and calls other functions to fill in other records. A buy might be affected by many events in the future.  PropagateDelete isn't used with buy because it isn't needed with a new node. There is no propagate routine in buy because it should never be called based on the way the system works.  A buy is only propagated during insert.
  
=cut
sub insert
{
  my $self = shift;
  if (!$self->isValid) { return undef; }
  my $eventRef = $self->nextSecurityEventRef();
  my $counter = $self->nextSecurityEventCounter($self->get('date'));
  $self->set('id', $eventRef);
  $self->set('counter', $counter);
    
  $self->{dbHandle}->write(
    sql => "INSERT INTO sec_event_tb (event_code, trans_date, counter, S, A, OS, NS, CC, ledger_code, event_ref, comment) "
    .                        "VALUES (?,          ?,          ?,       ?, ?, ?,  ?,  ?,  ?,           ?,         ?)",
    plug => [
      $self->get('type'),
      $self->get('date') ,
      $counter ,
      $self->get('oldStock') ,
      $self->get('newStock') ,  ###!!!! might be reversed
      $self->get('oldShares') ,
      $self->get('newShares') ,
      ($self->get('cost') / 100.0) ,
      '*',
      $eventRef ,
      $self->get('comment')
    ]
  );

  $self->insertIntoLedgers(date => $self->get('date'), counter => $counter, stock => $self->get('oldStock'));
  if (!$self->error) { $self->{dbHandle}->commit; }
  else { $self->{dbHandle}->rollback; }
  return $eventRef;
}

#######################################
#left this code out of new version because I cant see any reason for it to happen.  Need think about this

   # swap the values in the nextSecObj object if they are backwards compared to the lot node object
#     if ($lotNodeObj->subAccountNum ne $nextSecObj->subAccountNum)
#     {
#       # swap sub-account numbers
#       my $tempSubAccountNum = $nextSecObj->subAccountNum2;
#       $nextSecObj->subAccountNum2($nextSecObj->subAccountNum);
#       $nextSecObj->subAccountNum($tempSubAccountNum);
#       # swap new shares with old shares
#       my $tempOldShares = $nextSecObj->oldShares;
#       $nextSecObj->oldShares($nextSecObj->newShares);
#       $nextSecObj->newShares($tempOldShares);
#       # swap cost percentage
#       $nextSecObj->costPercentage(1.0 - $nextSecObj->costPercentage);
#     }
##########################################

=item boolean propagate(LotNode)

  See Investments::Objects::Event::propagate

=cut
sub propagate
{
  my $self = shift;
  my $old = shift;
  my $new1 = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $old->get('ledgerCode'));
  my $new2 = Portal::Accounting::Investments::Objects::LotNode->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $old->get('ledgerCode'));
  
  #calculate new lot node data from old lot and data from this eventRef
  my $newLotCost = $self->get('cost')/100.00 * $old->get('cost');
  my $oldLotCost = $old->get('cost') - $newLotCost;
  # calculate new lot's number of shares
  my $newShares = $self->get('newShares') / $self->get('oldShares') * $old->get('shares');
  # get a new lot number
  my $newLotNumber = $self->nextLotNumber();
  if (!$self->error)
  {
    # create a transaction to generate a new lot moving the cost of the old lot to the new lot
    my $transobj = Portal::Accounting::Objects::JournalEntry->new(dbHandle => $self->{dbHandle}, langObj => $self->{langObj}, ledgerCode => $old->get('ledgerCode') );
    $transobj->set('journal' , $self->{Journal}); 
    $transobj->set('date' , $self->get('date') );
    $transobj->addRow(
      subEntity => $old->get('subEntity'),
      account => $old->get('account'),
      subAccount => $self->get('oldStock'),
      dorc => "false",
      amount => sprintf("%.2f", $newLotCost),
      comment => $self->get('newShares')." shares for lot $newLotNumber. ". $self->get('comment'),
    );
    $transobj->addRow(
      subEntity => $old->get('subEntity'),
      account => $old->get('account'),
      subAccount => $self->get('newStock'),
      dorc => "true",
      amount => sprintf("%.2f", $newLotCost),
      comment => $self->get('newShares')." shares for lot $newLotNumber. ". $self->get('comment'),
    );
    # insert transaction
    if ($transobj->error)
    {
      $self->error($transobj->errorMessage);
      return undef;
    }
    my $refNum = $transobj->insert();
    if ($transobj->error)
    {
      $self->error($transobj->errorMessage);
      return undef;
    }

    # create two new lot nodes, one for the new lot that is generated from the event and one for the old lot to decrease
    # its balances (shares, cost)
    $new2->populate(
      date => $self->get('date'),
      parent => $old->get('id'),
      event => $self->get('id'),
      journalRef => $transobj->get('id'),
      subEntity => $old->get('subEntity'),
      account => $old->get('account'),
      subAccount => $self->get('newStock'),
      lot => $newLotNumber,
      buyLot => $old->get('buyLot'),
      cost => $newLotCost,
      shares => sprintf("%.6f", $newShares),
    );
    $new1->populate(
      date => $self->get('date'),
      parent => $old->get('id'),
      event => $self->get('id'),
      journalRef => $transobj->get('id'),
      subEntity => $old->get('subEntity'),
      account => $old->get('account'),
      subAccount => $self->get('oldStock'),
      lot => $old->get('lot'),
      buyLot => $old->get('buyLot'),
      cost => $oldLotCost,
      shares => $old->get('shares'),
    );
    $new1->insert;
    if ($new1->error) { $self->error($new1->errorMessage); return undef; }
    $new2->insert;
    if ($new2->error) { $self->error($new2->errorMessage); return undef; }
    $self->SUPER::propagate($new1);
    if (!$self->error) { $self->SUPER::propagate($new2); }
  }
  return 1;
}

=item boolean delete

  Routine calls all of the neccessary functions to get rid of this event and all nodes affected by it in lot trees.
  
=cut
sub delete
{
  my $self = shift;
  return $self->SUPER::deleteEvents(@_);
}

=back

=cut

1;
__END__

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Objects::Base(3)

=cut