# StockSplit.pm - The Object Class that provides a StockSplit Object
#$Id: StockSplit.pm,v 1.14.4.3 2006/08/08 17:12:46 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2005 http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Investments::Objects::StockSplit;
use strict;
use Portal::Accounting::Investments::Objects::Event;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Investments::Objects::Event Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.14.4.3 $';

=head1 NAME

StockSplit

=head1 SYNOPSIS

  use Portal::Accounting::Investments::Objects::StockSplit;
  my $obj = Portal::Accounting::Investments::Objects::StockSplit->new();
  $obj->populate(date =>, . . .  );
  if ($obj->error)
  {
    die $errorMessage();
  }

=head1 DESCRIPTION

  StockSplit is the class used to manage StockSplit security events.  It contains forms
  for creating, editing (if this gets implemented), and deleting StockSplit events.
  It also contains the database functionality for inserting and deleting
  records.  As with all security events LotNode is used as part of the
  processes to create, delete, and insert data.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(dbHandle => , langObj =>, ledgerCode =>)

  Creates a new instance of the Portal::Accounting::Investments::Objects::StockSplit module.

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( ledgerCode => "", app => "Accounting", state => "Investments::Journal", @_ );

  # instantiate anything unique to this module
  $self->{dbHandle} = $args{dbHandle};
  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }
  # instantiate anything unique to this module
  my $me = "splt";
  my $l = $self->{langObj};
  my @date = $self->getDateArray;
  my $today = $self->getCurrentDate;

  $self->{defaultForm} = "main";
  $self->{data}->{main} = {
    "app" => { -Type => "hidden", -Value => $args{app} },
    "state" => { -Type => "hidden", -Value => $args{state} },
    "command" => { -Type => "hidden", -Value => "save" },
    "id" => { -Type => "hidden", },
    "counter" => { -Type => "hidden", },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, -DataType => "string" },
    "type" => { -Type => "hidden", -Label => $l->map($me), -Value => $me, -DataType => "string" },
    "date" => { -Label => $l->map("Date"), -Type => "date-picker", -DataType => "string",
      -year => $date[0], -Value => $today },
    "stock" => { -Label => $l->map("Stock"), -Type => "select", -DataType => "integer",
      -Options => $self->{stocks}      },
    "totalShares" => { -Type => "text", -DataType => "float", -Label => $l->map("Total Shares"), size => 8 },
    "oldShares" => { -Type => "text", -DataType => "float", -Label => $l->map('Shares'), size => 8},
  };
  $self->{data}->{report} = {
    "id" => { -Type => "hidden", },
    "counter" => { -Type => "hidden", },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, },
    "type" => { -Type => "text", -Label => $l->map($me), -Value => $me, -DataType => "string", -ReadOnly => 1, },
    "date" => { -Label => $l->map("Date"), -Type => "text", -DataType => "string",  -ReadOnly => 1, },
    "stock" => { -Label => $l->map("Stock"), -Type => "select", -DataType => "integer",  -ReadOnly => 1,
      -Options => $self->{stocks}      },
    "totalShares" => { -Type => "text", -DataType => "float", -Label => $l->map("Total Shares"), -ReadOnly => 1,},
    "oldShares" => { -Type => "text", -DataType => "float", -Label => $l->map('Shares'), -ReadOnly => 1,},
  };
  $self->{css}->{main} = <<"ENDOF";
ENDOF
  my $Submit = $l->map("Submit");
  $self->{template}->{main} = <<"ENDOF";
<form #FORMARGS#>
  #FORMERRORSTRING#
  #FI=app#
  #FI=state#
  #FI=command#
  #FI=counter#
  #FI=ledgerCode#
  <h4>#LABEL=type##FI=type#</h4>
  <table class="primary">
    <tr>
      <td>
        #LABEL=date# #FI=date# #LABEL=stock# #FI=stock#<br>
        #LANG=youWillHaveTotal# #FI=totalShares#
        #LANG=sharesForEvery# #FI=oldShares#
        #LANG=thatYouOwn#.
      </td>
    </tr>
  </table>
  <input type="submit" name="submit" value="#LANG=Submit#" onclick="this.disabled = true;">
</form>
ENDOF
  $self->{template}->{report} = <<"ENDOF";
<table id="i#VALUE=id#" class="report">
  <tr>
    <td class="date">#FIELD=date#</td>
    <td class="event"><a title="#LABEL=type#">#FIELD=type#</a></td>
    <td class="detail">#FIELD=totalShares# #LANG=for# #FIELD=oldShares#</td>
$self->{deleteString}   
  </tr>
</table>
ENDOF
  
  $self->{javascript}->{main} = <<"ENDOF";
ENDOF
  $self->{profile}->{main} = {
    required => [ qw( date oldShares totalShares stock ) ],
    optional => [ qw(counter) ],
    constraints => {
         date => qr/^\d{4}-\d{2}-\d{2}$/,
         oldShares => 'RE_num_real',
         totalShares => 'RE_num_real',
         stock => qr/^\d+$/,
         counter => qr/^\d+$/,
       },
  };
  $self->{profile}->{report} = {
    required => [ qw( date oldShares totalShares stock ) ],
    optional => [ qw(counter) ],
    constraints => {
         date => qr/^\d{4}-\d{2}-\d{2}$/,
         oldShares => 'RE_num_real',
         totalShares => 'RE_num_real',
         stock => qr/^\d+$/,
         counter => qr/^\d+$/,
       },
  };

  $self->{INSERTorder}[0] = [ qw( event_code trans_date counter s os ns ledger_code event_ref comment ) ] ; #insert order for db column names
  
  $self->{db2data}[0] = {
    event_code => 'type',
    trans_date => 'date',
    os => 'oldShares',
    s => 'stock',
    ns => 'totalShares',
    ledger_code => 'ledgerCode',
    event_ref => 'id',
  };  #map database columns to ->data hash, not specified assumes same name
  $self->{data2db}[0] = {reverse %{$self->{db2data}[0]}};

  $self->{SELECT}[0] = "SELECT ##COLUMNS## FROM $self->{table} WHERE event_ref = #FIELD=id#";
#  $self->{INSERT}[0] = "INSERT INTO $self->{table} (##COLUMNS##) VALUES (##VALUES##)";
#  $self->{UPDATE}[0] = "UPDATE $self->{table} SET ##UPDATE## WHERE event_ref = #FIELD=id#";
  $self->{DELETE}[0] = "DELETE FROM $self->{table} WHERE event_ref = #FIELD=id#";
  
  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item boolean insert

  Inserts the record for this event into database and calls other functions to fill in other records. A buy might be affected by many events in the future.  PropagateDelete isn't used with buy because it isn't needed with a new node. There is no propagate routine in buy because it should never be called based on the way the system works.  A buy is only propagated during insert.
  
=cut
sub insert
{
  my $self = shift;
  if (!$self->isValid) { return undef; }
  my $counter = $self->nextSecurityEventCounter($self->get('date'));
  my $eventRef = $self->nextSecurityEventRef();
  $self->set('id', $eventRef);
  $self->set('counter', $counter);
  $self->{dbHandle}->write(
    sql =>
      "INSERT INTO sec_event_tb (event_code, trans_date, counter, S, OS, NS, ledger_code, event_ref) "
      . "VALUES (?, ?, ?, ?, ?, ?, ?, ?)",
    plug => [
      $self->get('type'),
      $self->get('date') ,
      $counter ,
      $self->get('stock') ,
      $self->get('oldShares') ,
      $self->get('totalShares') ,
      '*',
      $eventRef,
    ]
  );

  if (!$self->dbHandle->error)
  {
    ##get all affected lots
    ##loop and propagate on each one
    $self->insertIntoLedgers(date => $self->get('date'), counter => $counter, stock => $self->get('stock'));
  }
  else { $self->error($self->dbHandle->errorMessage); }
      
  if (!$self->error) { $self->{dbHandle}->commit; }
  else { $self->{dbHandle}->rollback; }
  return $eventRef;
}

=item boolean propagate(LotNode)

  See Investments::Objects::Event::propagate

=cut
sub propagate
{
  my $self = shift;
  my $lotNodeObj = shift;
  my $lc = $lotNodeObj->get('ledgerCode');
  my $totalShares = $self->get('totalShares') / $self->get('oldShares') * $lotNodeObj->get('shares');

    # insert a new lot node into the database
  my $newLotNodeObj = Portal::Accounting::Investments::Objects::LotNode->new(ledgerCode => $lc, langObj => $self->langObj, dbHandle => $self->dbHandle );
  $newLotNodeObj->populate(
    date => $self->get('date'),
    parent => $lotNodeObj->get('id'),
    event => $self->get('id'),
    subEntity => $lotNodeObj->get('subEntity'),
    account => $lotNodeObj->get('account'),
    subAccount => $lotNodeObj->get('subAccount'),
    lot => $lotNodeObj->get('lot'),
    buyLot => $lotNodeObj->get('buyLot'),
    cost => $lotNodeObj->get('cost') ,
    shares => sprintf("%.6f", $totalShares),
  );
  $newLotNodeObj->insert;
  if ($newLotNodeObj->error)
  {
    $self->error($newLotNodeObj->errorMessage);
    return undef;
  }

  return $self->SUPER::propagate($newLotNodeObj);
}

#StockDividend
#       $self->{dbHandle}->write(
#         sql => "INSERT INTO sec_event_tb (event_code, trans_date, counter, S, OS, NS, ledger_code, event_ref, comment) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
#         plug => [ $secObj->eventCode , $secObj->calcDate , $secObj->transDate , $counter , $secObj->subAccountNum , $secObj->oldShares , $secObj->totalShares , '*', $secObj->eventRefNum
#         ]
#       );

=item boolean delete

  Routine calls all of the neccessary functions to get rid of this event and all nodes affected by it in lot trees.
  
=cut
sub delete
{
  my $self = shift;
  return $self->SUPER::deleteEvents(@_);
}

=back

=cut

1;
__END__

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Objects::Base(3)

=cut