# CloseOut.pm - The Object Class that provides a CloseOut Object
#$Id: CloseOut.pm,v 1.23.2.1 2007/02/25 03:15:43 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2003 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::CloseOut;
use strict;
use Portal::Accounting::Objects::Base;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.23.2.1 $';

=head1 NAME

CloseOut - Object used to build a CloseOut Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::CloseOut;
  my $CloseOutObj = Portal::Accounting::Objects::CloseOut->new(langObj =>, dbHandle =>, ledgerCode =>,   );
  if ($CloseOutObj->error())
  {
    die $CloseOutObj->errorMessage();
  }

=head1 DESCRIPTION

CloseOut is a CloseOut class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new( )

    Creates a new instance of Portal::Accounting::Objects::CloseOut.
    See Portal::Accounting::Objects::Base(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( ledger => undef, @_ );

  if ($self->error)
  {
    $self->error();
    return $self;
  }

  # instantiate anything unique to this module
  my $ph = $self->{langObj};
  $self->{ledger} = $args{ledger};
  my $ledgerCode = $args{ledger}->get('code');
  my $submit = { -Type => "submit", -Value => $ph->map("Submit"), onclick => "this.disabled = true;", };
  my $message;
  if (defined $a->{values} && scalar @{$a->{values}} == 0)
  {
    $message = "<a href=\"?app=Accounting&state=Accounts&command=display&ledgerCode=$ledgerCode\">".$ph->map('noEquityAccounts')."</a>\n";
    $submit = { -Type => "hidden", -Value => $ph->map("Submit") };
  }

  my @dates = $self->{ledger}->getQuarterEnds(-1);
  my %dates;
  foreach my $date (@dates)
  {
    $dates{$date} = $date;
  }

  $self->{data}->{main} = {
    "date" => { -Label => $ph->map("close_lock"), -Type => "date-picker",
      -year => $self->{ledger}->get('fiscalYear') , -DataType => "date",
      },
    "businessType" => { -Label => $ph->map("Business Type"), -Type => "text", -ReadOnly => 1, -DataType => "string",
      -Value => $self->{ledger}->businessTypeName,
    },
    "command" => { -Type => "hidden", -Value => "closeOut" },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledger}->get('code') },
    "app" => { -Type => "hidden", -Value => "Accounting" },
    "state" => { -Type => "hidden", -Value => "Ledger" },
    "lastClose" => { -Label => $ph->map("Last Close Out"), -Type => "text", -ReadOnly => 1, -DataType => "date", -Value => $self->{ledger}->lastClose },
    "dates" => { -Label => $ph->map("Autofill"), -Type => "select", size => 4, -DataType => 'date', -Options => $self->{formObj}->createSelectOptions(\%dates),
      onchange => "enterDate(this);"
      },
    "submit" => $submit,
  };

  $self->{javascript}->{main} = <<"ENDOF";
  function enterDate(me)
  {
    if (me.selectedIndex == -1)
    {
      return;
    }
    me.form.date.value = me.options[me.selectedIndex].value;
  }
ENDOF
  $self->{template}->{main} = <<"ENDOF";
  <p>#FORMERRORSTRING#
  <form #FORMARGS#>
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=ledgerCode#
  $message
  <div class="secondary" style="float: right;">
    #LABEL=businessType#: #FIELD=businessType#<br>
    #LABEL=lastClose#: #FIELD=lastClose#
  </div>
  <div class="primary">
    #LANG=closeoutExplain#
    <table cellpadding="15">
      <tr valign="top">
        <td>
          #LABEL=date#<br>
          #FIELD=date#<br>
        </td>
        <td>
          #LABEL=dates#<br>
          #FIELD=dates#
        </td>
      </tr>
    </table>
    <p>#FIELD=submit#<input type="reset"></p>
  </div>
    <div class="tertiary" style="padding: .5em;">
      #LANG=closeNote#
    </div>
  </form>
ENDOF

  $self->{profile}->{main} = {
    required => [ qw( date ) ],
    constraints  =>
        {
            date   => qr/^\d{4}-\d{2}-\d{2}$/,
        },
  };
  if ($self->{formObj}->error) { $self->error($self->{formObj}->errorMessage); }
  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->error();
    return 0;
  }

  # validate our parameters.
  my $date = $self->get('date');
  if ($date =~ /^(\d{4})-(\d{1,2})-(\d{1,2})$/)
  {
    $self->set('date', "$1-".sprintf("%02d",$2)."-".sprintf("%02d",$3) );
  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

sub isNew
{
  return 1;
}

=item string update(commit =>  )

 Closes out ledger on the given date
 Returns true or false
 undef on errors and sets the errorMessage.
 commit is set to 1 by default.  This will commit all work done to
 the database at the end of the function by default.  To override, set
 commit to 0.  You will then need to commit the transaction(s) to the database.

=cut

sub update
{
  my $self = shift;
  my %args = ( commit => 1, @_ );
  my $ledger = $self->{ledger};
  my $commit = $args{commit};
  my $closeoutDate = $self->get('date');
  my $fromDate;
  my $sth;

  my $currentFiscalYear = $ledger->workingYear;
  $fromDate = $ledger->start(month => 1, fiscalYear => $currentFiscalYear);
  if ($ledger->error)
  {
    $self->error($ledger->errorMessage);
    return undef;
  }

  # verify that the closeoutDate has not already been used or that the ledger has not been closed out after closeoutDate
  my $foo;
  my $bar;
  ($foo = $closeoutDate) =~ s/-//g;
  ($bar = $fromDate) =~ s/-//g;
  if ($foo < $bar)
  {
    $self->error(sprintf $self->{langObj}->map("closeoutDateInvalid"), $closeoutDate, $fromDate);
    return undef;
  }

  ($foo = $closeoutDate) =~ s/-//g;
  ($bar = $ledger->lastClose()) =~ s/-//g;
  if ($foo <= $bar)
  {
    $self->error(sprintf $self->{langObj}->map("closeoutDateInvalid"), $closeoutDate, $bar);
    return undef;
  }

  my %years;
  $years{$currentFiscalYear} = $closeoutDate;
  #compare closeoutdate with working year end.
  my $index = $currentFiscalYear;
  my $w = $ledger->end(month => 12, fiscalYear => $index);
  if ($closeoutDate <= $w) { $years{"$index"} = $closeoutDate; }
  else
  {
    while ($closeoutDate >= $w)
    {
      #  put w in $years
      $years{"$index"} = $w;
      #  add one year to w
      $w = $ledger->end(month => 12, fiscalYear => ++$index);
    }
    $index--;
    #check last entry in case $closeout is not at end of year
    $years{"$index"} = $closeoutDate;
  }

  # verify that debits == credits for all transactions in the system
  my $systemBalances = $ledger->doesSystemBalance(beginDate => $ledger->lastClose, endDate => $closeoutDate);
  if ($ledger->error) { $self->error($ledger->errorMessage); return undef; }
  if ($systemBalances > 0)
  {
    $self->error($self->{langObj}->map("debitsGreaterThanCredits"));
    return undef;
  }
  elsif ($systemBalances < 0)
  {
    $self->error($self->{langObj}->map("creditsGreaterThanDebits"));
    return undef;
  }
  my $code = $ledger->get('code');
  foreach my $y (sort keys %years)
  {
    my $yearEnd = $ledger->end(month => 12, fiscalYear => $y);
    my $sql;
    #UPDATE known years tb
    if ($years{$y} eq $yearEnd)
    {
      #update current year
      my $sql = "UPDATE known_years_tb set closed = 'true' WHERE ledger_code = ? AND year <= ?";
      $sth = $self->{dbHandle}->write(sql => $sql, plug => [ $code, $y] );
      if ($self->{dbHandle}->error)
      {
        $self->error($self->{langObj}->map("error") . "<br>$sql<br>\n" . $self->{dbHandle}->errorMessage);
        return undef;
      }
      #check for next year record
      $sql = "SELECT year FROM known_years_tb WHERE ledger_code = ? AND year > ?";
      $sth = $self->{dbHandle}->read(sql => $sql, plug => [$code, $y ] );
      if ($self->{dbHandle}->error)
      {
        $self->error($self->{langObj}->map("error") . "<br>$sql<br>\n" . $self->{dbHandle}->errorMessage);
        return undef;
      }
      #if no record existed insert one for the next year
      if (scalar @{$sth->fetchall_arrayref} == 0)
      {
        $sql = "INSERT INTO known_years_tb (ledger_code, year, closed) VALUES (?, ?, false)";
        $sth = $self->{dbHandle}->write(sql => $sql, plug => [ $code, ($y + 1) ] );
        if ($self->{dbHandle}->error)
        {
          $self->error($self->{langObj}->map("error") . "<br>$sql<br>\n" . $self->{dbHandle}->errorMessage);
          return undef;
        }
      }
    }
    #insert closeout information
    my $close = "INSERT INTO closeouts (ledger_code, date) VALUES (?, ?)";
    $sth = $self->{dbHandle}->write(sql => $close, plug => [$code, $years{$y} ] );
    if ($self->{dbHandle}->error)
    {
      $self->error($self->{langObj}->map("error") . "<br>$close<br>\n" . $self->{dbHandle}->errorMessage);
      return undef;
    }
  }

  if ($commit)
  {
    $sth = $self->{dbHandle}->commit;
    if ($self->{dbHandle}->error)
    {
      $self->error($self->{langObj}->map("error") . "<br>COMMIT<br>\n" . $self->{dbHandle}->errorMessage);
      return undef;
    }
  }
  my $p = $self->langObj;
  return $p->map("Success") ." ". $p->map("Close Books") . " " . $closeoutDate . " " . $ledger->get('code');
}

=item string print()

  This routine returns a text string that represents the data in this object.

=cut

=item string printXML(fullXML bool, indent int)

 This routine returns an XML snippet for this object for use in such
 things as exporting data

 fullXML - defaults to 0 (false).  If true, we output the xml header tag.
 indent - defaults to 2.  Specifies how many spaces in our top level tag
          needs to be.  If fullXML is true, then we force 0, otherwise we
          use what the user specifies.

=cut

=item string printHTML()

  This routine returns a string of html that represents this object.

=cut

=item string printForm( prefix => )

  This routine returns xhtml form elements that can be used to edit this object.
  A prefix can be used to uniquely identify this form from others of the same
  type.  The names of form elements can be obtained in a hash:

  use Portal::Accounting::Objects::Base;
  my %elements = %Portal::Accounting::Objects::CloseOut::Elements;

  !!Make a copy of the hash as shown above--NOT a refence.
  You would generally place your input from the user within the Elements hash.
  Then you can pass the Elements hash to the new method
  of a CloseOut object instead of individual arguments.

  foreach my $prefix (%prefixes)
    foreach $index (key %Elements)
      $tempHash{$index} = $input->{$prefix . $index}
    my object = Portal::Accounting::Objects::CloseOut->new(%tempHash);

=cut

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal::Accounting::Objects::Base(3)

=cut
