# Demo.pm - The Object Class that provides a Demo Object
#$Id: Demo.pm,v 1.5 2006/04/05 02:57:50 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2003 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::Demo;
use strict;
use Portal::Accounting::Objects::BaseL;
use Portal::Accounting::Objects::Account;
use Portal::Accounting::Objects::SubAccount;
use Portal::Accounting::Investments::Objects::Stock;
use Portal::Accounting::Objects::SubEntity;
use Portal::Accounting::Objects::Ledger;
use Portal::Accounting::Objects::JournalEntry;
use Date::Calc qw(Today);
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::BaseL Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.5 $';

=head1 NAME

Demo - Object used to build a Demo Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::Demo;
  my $DemoObj = Portal::Accounting::Objects::Demo->new(   );
  if ($DemoObj->error())
  {
    die $DemoObj->errorMessage();
  }

=head1 DESCRIPTION

Demo is a Demo class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new( )

    Creates a new instance of Portal::Accounting::Objects::Demo.
    See Portal::Accounting::Objects::Base(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( ledgerCode => '', dbHandle => undef, @_ );
  $self->{new} = 1;
  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{dbHandle} = $args{dbHandle};
  $self->{object} = "Portal::Accounting::Objects::JournalEntry";
  my @date = Today();
  my $l = $self->{langObj};
  if (! defined $l) { $self->error('langObj is screweed!'); return $self; }
  my $between = $l->map('between');
  my $acc = Portal::Accounting::Objects::Account->new(langObj => $l, dbHandle => $self->{dbHandle}, ledgerCode => $args{ledgerCode});
  my $sacc = Portal::Accounting::Objects::SubAccount->new(langObj => $l, dbHandle => $self->{dbHandle}, ledgerCode => $args{ledgerCode});
  my $st = Portal::Accounting::Investments::Objects::Stock->new(langObj => $l, dbHandle => $self->{dbHandle}, ledgerCode => $args{ledgerCode});
  my $se = Portal::Accounting::Objects::SubEntity->new(langObj => $l, dbHandle => $self->{dbHandle}, ledgerCode => $args{ledgerCode});
  if ($acc->error) { $self->error($acc->errorMessage); }
  if ($sacc->error) { $self->error($sacc->errorMessage); }
  if ($st->error) { $self->error($st->errorMessage); }
  if ($se->error) { $self->error($se->errorMessage); }

  my $je = Portal::Accounting::Objects::JournalEntry->new(langObj => $l, dbHandle => $self->{dbHandle}, ledgerCode => $args{ledgerCode});
  if ($sacc->error) { $self->error("<p>Error during object creation!</p>".$sacc->errorMessage."\n"); return $self; }
  my $all = $l->map('ALL');
  $acc->set('meta', '5');
  my $ex = $acc->selectOptionsByMeta();
  $acc->set('meta', '4');
  my $r = $acc->selectOptionsByMeta();
  $acc->set('meta', '3');
  my $e = $acc->selectOptionsByMeta();
  $acc->set('meta', '2');
  my $l = $acc->selectOptionsByMeta();
  $acc->set('meta', '1');
  $acc->set('type', '0');
  my $b = $acc->selectOptionsByType();
  $acc->set('type', '3');
  my $i = $acc->selectOptionsByType();
  my $s = $se->selectOptions;
  if ($acc->error) { $self->error($acc->errorMessage); }
  if ($sacc->error) { $self->error($sacc->errorMessage); }
  if ($st->error) { $self->error($st->errorMessage); }
  if ($se->error) { $self->error($se->errorMessage); }
  foreach my $me ($b, $i, $l, $e, $r, $ex, $s)
  {
    if (not defined $me) { $me = { names => [], values => []}; }
  }
  $self->{data}->{main} = {
    "app" => { -Type => "hidden", -Value => "Accounting" },
    "state" => { -Type => "hidden", -Value => "JournalEntry" },
    "command" => { -Type => "hidden", -Value => "demo" },
    "confirmDemo" => { -Type => "hidden", -Value => "1" , -DataType => 'int'},
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode},},
    "description" => { -Type => "text", -DataType => "string", -Value => 'My Water Co.', -Label => "Payee" },
    "subEntities" => { -Label => "Sub Entity", -Type => "select", -DataType => "int",
      -Options => $s,
      -Buttons => [ "SelectAll", "ToggleSelect" ],
    },
    "bank" => { -Label => "Bank", -Type => "select", -DataType => "int",
      -Options => $b,
      -Buttons => [ "SelectAll", "ToggleSelect" ],
    },
    "investment" => { -Label => "Investement", -Type => "select", -DataType => "int",
      -Options => $i,
      -Buttons => [ "SelectAll", "ToggleSelect" ],
    },
    "liability" => { -Label => "Liability", -Type => "select", -DataType => "int",
      -Options => $l,
      -Buttons => [ "SelectAll", "ToggleSelect" ],
    },
    "equity" => { -Label => "Equity", -Type => "select", -DataType => "int",
      -Options => $e,
      -Buttons => [ "SelectAll", "ToggleSelect" ],
    },
    "revenue" => { -Label => "Revenue", -Type => "select", -DataType => "int",
      -Options => $r,
      -Buttons => [ "SelectAll", "ToggleSelect" ],
    },
    "expense" => { -Label => "Expense", -Type => "select", -DataType => "int",
      -Options => $ex,
      -Buttons => [ "SelectAll", "ToggleSelect" ],
    },
    "comment" => { -Label => "Comment", -Type => "text",    -Value => "Foo bar <b> test </b>",
      size => 25, -DataType => "string",
    },
    "amount1" => {-Type => "calculator", size => 10, -Value => "100.00", -Label => "Amount",
      -DataType => "float", -undoButton => 0,
    },
  };
   $self->{template}->{main} = <<"ENDOF";
<p><b>Choose data from this list and enter data here.  Then click the submit button if you are sure that you want to enter demo data into these books.</b></p>
If the select fields are empty you need to <a href="?app=$self->{app}&state=Accounts&command=display&ledgerCode=$args{ledgerCode}">assign Accounts</a> to this ledger.  You may have to create some accounts of the types required for this screen.
<form #FORMARGS#>
  #FORMERRORSTRING#
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=ledgerCode#
  #FIELD=confirmDemo#
  
<div style="clear: both; float: left; width: 10em; ">#LABEL=bank#</div><div> #FIR=bank#</div>
<div style="clear: both; float: left; width: 10em; ">#LABEL=investment#</div><div> #FIR=investment#</div>
<div style="clear: both; float: left; width: 10em; ">#LABEL=liability#</div><div> #FIR=liability#  </div>
<div style="clear: both; float: left; width: 10em; ">#LABEL=equity#</div><div> #FIR=equity#        </div>
<div style="clear: both; float: left; width: 10em; ">#LABEL=expense#</div><div> #FIR=expense#      </div>
<div style="clear: both; float: left; width: 10em; ">#LABEL=revenue#</div><div> #FIR=revenue#      </div>
<p><b>These are optional and will be used in one of the sets of entries.</b></p>
<div style="clear: both; float: left; width: 10em; ">#LABEL=description#</div><div> #FI=description#</div>
<div style="clear: both; float: left; width: 10em; ">#LABEL=subEntities#</div><div>       #FI=subEntities#</div>
<div style="clear: both; float: left; width: 10em; ">#LABEL=amount1#</div><div>        #FI=amount1#</div>
<div style="clear: both; float: left; width: 10em; ">#LABEL=comment#</div><div> #FI=comment#</div>

  <input type="submit" name="submit" value="#LANG=Submit#" />
</form>
ENDOF
  $self->{javascript}->{main} = <<"ENDOF";
ENDOF
  $self->{profile}->{main} = {
    required => [ qw( bank liability revenue expense equity investment ) ],
    optional => [ qw( amount1 comment description subAccounts accounts subEntities stocks) ],
    constraints => {
      amount1 => qr/^\d+\.?\d*$/,
      accounts => qr/^\d+$/,
      subAccounts => qr/^-?\d+$/,
      stocks => qr/^\d+$/,
      bank => qr/^\d+$/,
      liability => qr/^\d+$/,
      revenue => qr/^\d+$/,
      expense => qr/^\d+$/,
      equity => qr/^\d+$/,
      investment => qr/^\d+$/,
    },
  };

  # do anything else you might need to do.

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

sub printReport
{
  my $self = shift;
  my $doc = HTMLObject::Normal->new;
  my $year;
  my $lc = $self->get('ledgerCode');
  my $W;
  my  $P;
  my  $E;
  my  $I;
   
  if ($year !~ /^\d{4}$/) { my @year = Today; $year = $year[0];}
  my $expense = $self->get('expense');
  my $revenue = $self->get('revenue');
  my $bank = $self->get('bank');
  my $investment = $self->get('investment');
  my $equity = $self->get('equity');
  my $liability = $self->get('liability');
#setup objects
  $doc->print(<<"ENDOF");
<p>using accounts:<br>
bank = $bank<br>
investment = $investment<br>
liability = $liability<br>
equity = $equity<br>
expense = $expense<br>
revenue = $revenue<br>
</p><hr>
ENDOF
  #loop 12 times for inserting 12 months.  start with 2 bc 1 is already set
  for (my $month = 1; $month < 13; $month++)
  {
    my $elec = Portal::Accounting::Objects::JournalEntry->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle}, ledgerCode => $lc);
    my $water = Portal::Accounting::Objects::JournalEntry->new(langObj => $self->{langObj}, dbHandle => $self->dbHandle, ledgerCode => $lc);
    my $pay = Portal::Accounting::Objects::JournalEntry->new(langObj => $self->{langObj}, dbHandle => $self->dbHandle, ledgerCode => $lc);
    my $inv = Portal::Accounting::Objects::JournalEntry->new(langObj => $self->{langObj}, dbHandle => $self->dbHandle, ledgerCode => $lc);
  #populate objects
    $elec->set('payee', 'Some Electric Co');
    $elec->set('remember', 'false');
    $elec->set('date',"$year-$month-10");
    $elec->addRow(counter => 0,
      account => $expense,
      amount => "50.00",
      dorc => "true",
      comment => "a month $month",
    );
    if ($elec->error) { $doc->print($elec->errorMessage); }
    $elec->addRow(counter => 1,
      account => $bank,
      dorc => "false",
      amount => "50.00",
      comment => "b month $month",
    );
    if ($elec->error) { $doc->print($elec->errorMessage); }
    if (! $elec->isValid)
    {
      $doc->print($elec->errorMessage);
      $doc->print("l:".$lc);
#       $doc->print("<br>0: ".$elec->{rowObjects}[0]->get('dorc')."<br>1: ".$elec->{rowObjects}[1]->get('dorc')."<br>");
#       $doc->print("0: ".$elec->{rowObjects}[0]->get('amount')."<br>1: ".$elec->{rowObjects}[1]->get('amount')."<br>");
#       $doc->print("0: ".$elec->{rowObjects}[0]->get('d_amount')."<br>1: ".$elec->{rowObjects}[1]->get('d_amount')."<br>");
#       $doc->print("0: ".$elec->{rowObjects}[0]->get('c_amount')."<br>1: ".$elec->{rowObjects}[1]->get('c_amount')."<br>");
#       $doc->print("0: ".$elec->{rowObjects}[0]->get('account')."<br>1: ".$elec->{rowObjects}[1]->get('account')."<br>");
      return $doc;
    }

    #water object
    $water->set('payee', $self->get('description'));
    $water->set('date', "$year-$month-15");
    $water->addRow(counter => 0,
      comment => "month $month" . $self->get('comment'),
      dorc => "true",
      account => $expense,
      amount => $self->get('amount1'),
      subEntity => $self->get('subEntities'),
    );
    $water->addRow(counter => 1,
      comment => "month $month" . $self->get('comment'),
      account => $bank,
      dorc => "false",
      amount => $self->get('amount1'),
      subEntity => $self->get('subEntities'),
    );
    if (! $water->isValid)
    {
      $doc->print($water->errorMessage);
      $doc->print("<br>0: ".$water->{rowObjects}[0]->dorc."<br>1: ".$water->{rowObjects}[1]->get('dorc')."<br>");
      $doc->print("0: ".$water->{rowObjects}[0]->get('amount')."<br>1: ".$water->{rowObjects}[1]->get('amount')."<br>");
      $doc->print("0: ".$water->{rowObjects}[0]->get('d_amount')."<br>1: ".$water->{rowObjects}[1]->get('d_amount')."<br>");
      $doc->print("0: ".$water->{rowObjects}[0]->get('c_amount')."<br>1: ".$water->{rowObjects}[1]->get('c_amount')."<br>");
      $doc->print("0: ".$water->{rowObjects}[0]->get('account')."<br>1: ".$water->{rowObjects}[1]->get('account')."<br>");
      return $doc;
    }

    #pay object
    $pay->set('payee', 'Revenue');
    $pay->set('remember', 'false');
    $pay->set('date', "$year-$month-1");
    $pay->addRow(counter => 0,
      comment => "month $month",
      account => $revenue,
      amount => "1000.00",
      dorc => "false",
    );
    $pay->addRow(counter => 1,
      comment => "month $month",
      account => $bank,
      dorc => "true",
      amount => "1000.00",
    );
    if (! $pay->isValid)
    {
      $doc->print($pay->errorMessage);
      $doc->print("<br>0: ".$pay->{rowObjects}[0]->dorc."<br>1: ".$pay->{rowObjects}[1]->get('dorc')."<br>");
      $doc->print("0: ".$pay->{rowObjects}[0]->get('amount')."<br>1: ".$pay->{rowObjects}[1]->get('amount')."<br>");
      $doc->print("0: ".$pay->{rowObjects}[0]->get('d_amount')."<br>1: ".$pay->{rowObjects}[1]->get('d_amount')."<br>");
      $doc->print("0: ".$pay->{rowObjects}[0]->get('c_amount')."<br>1: ".$pay->{rowObjects}[1]->get('c_amount')."<br>");
      $doc->print("0: ".$pay->{rowObjects}[0]->get('account')."<br>1: ".$pay->{rowObjects}[1]->get('account')."<br>");
      return $doc;
    }

    #invest object
    $inv->set('payee' , 'Dividend Revenue'    );
#    $inv->set('remember' , 'false'    );
    $inv->set('date' , "$year-$month-1"    );
    $inv->addRow(counter => 0,
      comment => "month $month",
      account => $revenue,
      dorc => "false",
      amount => "200.00",
    );
    if ($inv->error) { $doc->print($inv->errorMessage); return $doc; }
    $inv->addRow(counter => 1,
      comment => "month $month",
      account => $investment,
      amount => "200.00",
      dorc => "true",
    );
    if (! $inv->isValid || $inv->error)
    {
      $doc->print($inv->errorMessage);
      $doc->print("<br>0: ".$inv->{rowObjects}[0]->get('dorc')."<br>1: ".$inv->{rowObjects}[1]->get('dorc')."<br>");
      $doc->print("0: ".$inv->{rowObjects}[0]->get('amount')."<br>1: ".$inv->{rowObjects}[1]->get('amount')."<br>");
      $doc->print("0: ".$inv->{rowObjects}[0]->get('d_amount')."<br>1: ".$inv->{rowObjects}[1]->get('d_amount')."<br>");
      $doc->print("0: ".$inv->{rowObjects}[0]->get('c_amount')."<br>1: ".$inv->{rowObjects}[1]->get('c_amount')."<br>");
      $doc->print("0: ".$inv->{rowObjects}[0]->get('account')."<br>1: ".$inv->{rowObjects}[1]->get('account')."<br>");
      return $doc;
    }

    my $ref;
    my $r;

    #insert current data into DB first
    $ref = $elec->insert(commit => 1);
    if ($elec->error)
    {
      $doc->print("<h4>Error Occurred while inserting Journal Entry Object</h4>" .$elec->errorMessage."<br>\n");
      return $doc;
    }
    $doc->print($ref.":".$elec->get("date")."<br>");
    $ref = $water->insert(commit => 1);
    if (not defined $ref)
    {
      $doc->print("<h4>Error Occurred while inserting Journal Entry Object</h4>" .$water->errorMessage."<br>\n");
      return $doc;
    }
    $doc->print($ref.":".$water->get("date")."<br>");
    $ref = $pay->insert(commit => 1);
    if (not defined $ref)
    {
      $doc->print("<h4>Error Occurred while inserting Journal Entry Object</h4>" .$pay->errorMessage."<br>\n");
      return $doc;
    }
    $doc->print($ref.":".$pay->get("date")."<br>");
    $ref = $inv->insert(commit => 1);
    if (not defined $ref)
    {
      $doc->print("<h4>Error Occurred while inserting Journal Entry Object</h4>" .$inv->errorMessage."<br>\n");
      return $doc;
    }
    $doc->print($ref.":".$inv->get("date")."<br>");
    $W = $water;
    $P = $pay;
    $E = $elec;
    $I = $inv;
  }
  $doc->print("Data Entered successfully for $year<br>\n");
  $doc->print('<hr>');
  $doc->print($W->printHTML);
  $doc->print('<hr>');
  $doc->print($E->printForm);
  $doc->print('<hr>');
  $doc->print($P->printXML);
  $doc->print('<hr><textarea rows="8" cols="80">');
  $doc->print($I->javascript);
  $doc->print('</textarea><hr>');
  
  return $doc;
}

sub populate
{
  my $self = shift;
  $self->{new} = 0;
  return $self->SUPER::populate(@_);
}

sub isNew
{
  my $self = shift;
  return $self->{new};
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal::Accounting::Objects::Base(3)

=cut
