#$Id: MO.pm,v 1.1 2006/05/12 14:35:46 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.

package Portal::Accounting::Payroll::Objects::State::MO;
use Portal::Accounting::Objects::BaseL;
use Portal::Accounting::Payroll::Objects::State;
use strict;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Payroll::Objects::State Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.1 $';

=head1 NAME

State - Object used to build a State Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Payroll::Objects::State::MO;
  my $obj = Portal::Accounting::Payroll::Objects::State::MO->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

State is a State class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new()

 Creates a new instance of the Portal::Accounting::Payroll::Objects::State::MO module.
 See Portal::Accounting::Objects::Base(3) for a listing of required arguments.

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = (ledgerCode => '', @_ );

  if ($self->error)
  {
    $self->error();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{name} = "Portal::Accounting::Payroll::Objects::State::MO";
  $self->{new} = 1;  #after populate it is no longer new
  $self->{table} = "state_payroll";
  $self->{id} = 'id';
  my $l = $self->{langObj};

  my $employees = $self->{EMPLOYEE}->selectOptions;
  my $employee;
  if (defined $args{employee}) { $employee = $args{employee}; }
  if ($self->{EMPLOYEE}->error) { $self->error($self->{EMPLOYEE}->errorMessage); }
  # do anything else you might need to do.
  $self->{data}->{main} = {
    "app" => { -Type => "hidden", -Value => "Accounting" },
    "state" => { -Type => "hidden", -Value => "Payroll::State" },
    "command" => { -Type => "hidden", -Value => "save" },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, -DataType => "string" },
    "states" => { -Type => 'text', -ReadOnly => 1, -DataType => "text", -Label => 'State', -Value => 'MO'},
    "id" => {-Type => 'hidden', -DataType => 'int', -Value => 0 },
    "employee" => { -Type => 'select', -Options => $employees, -DataType => "integer", -Label => 'Employee', -Value => $employee},
    "allow" => { -Type => 'text', -DataType => 'int', maxlength => 255, -Value => '0', -Label => 'Allowances',},
    "withHold" => { -Type => 'text', -DataType => 'float', maxlength => 255, -Value => '0.00', -Label => 'Extra Withholding'},
  };
  $self->{css}->{main} .= <<"ENDOF";
ENDOF
  $self->{template}->{main} = <<"ENDOF";
<form #FORMARGS#>
  #FORMERRORSTRING#
  Enter the Information claimed by this employee on the MO Federal Form W-4.
  #FIR=app#
  #FIR=state#
  #FIR=command#
  #FIR=ledgerCode#
  <div class="primary">
    <div>#FORMREQUIREDSTRING#</div>
    <div>#LABEL=id# #FIELD=id#</div>
    <div>#LABEL=states# #FIELD=states#</div>
    <div>#LABEL=employee# #FIELD=employee#</div>
    <div>#LABEL=allow# #FIELD=allow#</div>
    <div>#LABEL=withHold# #FIELD=withHold#</div>
    <input type="submit" value="Insert" >
  </div>
</form>  
ENDOF
  $self->{javascript}->{main} = <<"ENDOF";
ENDOF
  $self->{profile}->{main} = {
    required => [ qw( id states employee allow withHold ) ],
    optional => [ qw( ) ],
    constraints => {
    },
  };
  
  $self->{newOnly}[0] = { qw(id) };
  $self->{INSERTorder}[0] = [ qw( state employee) ] ;  #insert order for db column names
  $self->{db2data}[0] = { state => 'states' };  #map database columns to ->data hash, not specified assumes same name
  $self->{data2db}[0] = {reverse %{$self->{db2data}[0]}};
  $self->{SELECT}[0] = "SELECT ##COLUMNS## FROM $self->{table} WHERE id = '#FIELD=id#'";
  $self->{INSERT}[0] = "INSERT INTO $self->{table} (##COLUMNS##, key, value) VALUES (##VALUES##, 'allow', '#FIELD=allow#' )";
  $self->{UPDATE}[0] = "UPDATE $self->{table} SET ##UPDATE## WHERE $self->{id} = '#FIELD=$self->{id}#'";
  $self->{INSERTorder}[1] = [ qw( state employee) ] ;  #insert order for db column names
  $self->{db2data}[1] = { state => 'states' };  #map database columns to ->data hash, not specified assumes same name
  $self->{data2db}[1] = {reverse %{$self->{db2data}[0]}};
  $self->{SELECT}[1] = "SELECT ##COLUMNS## FROM $self->{table} WHERE id = '#FIELD=id#'";
  $self->{INSERT}[1] = "INSERT INTO $self->{table} (##COLUMNS##, key, value) VALUES (##VALUES##, 'withHold', '#FIELD=withHold#' )";
  $self->{UPDATE}[1] = "UPDATE $self->{table} SET ##UPDATE## WHERE $self->{id} = '#FIELD=$self->{id}#'";

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut
sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  if ($self->get('ledgerCode') eq '') { $self->missing('ledgerCode'); }
  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

sub isNew { my $self = shift; return ($self->get('id') == 0?1:0); }

=back

=cut

1;
__END__

=head1 NOTE

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Accounting::Objects::Base(3)

=cut
