# Payroll.pm - The Object Class that provides a Payroll Object
#$Id: State.pm,v 1.1 2006/05/12 14:35:46 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
# Created by Xperience, Inc.

package Portal::Accounting::Payroll::State;
use strict;
use Portal::Accounting::BaseL;
use Portal::Accounting::Payroll::Objects::State;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::BaseL Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.1 $';

# global variables.

# modify to represent the commands this state provides.
my %commands = (
  display => "Payroll",
  new => "Payroll",
  save => "Payroll",
);

=head1 NAME

Payroll - Object used to build a Payroll for accounting package

=head1 SYNOPSIS

  use Portal::Accounting::Payroll;
  my $obj = Portal::Accounting::Payroll->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

This module contains payroll tools.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(codeVersion, release, DBVersion)

 Creates a new instance of the Portal::Accounting::Payroll module.
 See Portal::Accounting::Base(3) for a listing of required arguments.

 Instantiates an instance of the Payroll State. !!Anything passed in here should be
 data that never changes from one run of a command to another such as database handles
 portalSession? input? cookieObj? dont fit that criterion

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module

  # do validation
  if (!$self->Portal::Accounting::Payroll::State::isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.
  $self->{commands} = \%commands;
  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }
  return 1;
}

=item HTMLObject display()

    Implements the default screen for this state.

=cut
sub c_display
{
  my $self = shift;
  $self->{OBJECT} = Portal::Accounting::Payroll::Objects::State->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});
  if ($@) { $self->error($@); return undef;  }
  if ($self->{OBJECT}->error) { $self->error($self->{OBJECT}->errorMessage); return undef;  }
  my $doc = $self->SUPER::c_display(@_);
#  $doc->print(css => ['form[name="new"] {display: none; }']);
  return $doc;
}

sub c_new
{
  my $self = shift;
  my $doc = $self->prepDoc;
  if ($self->{input}->{states} ne '' && $self->{input}->{employee} ne '' )
  {
    eval("use Portal::Accounting::Payroll::Objects::State::$self->{input}->{states};");
    eval('$self->{OBJECT2} = Portal::Accounting::Payroll::Objects::State::'.$self->{input}->{states}.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode}, employee => $self->{input}->{employee});');
    $doc->print($self->{OBJECT2}->printForm);
    if ($self->{OBJECT2}->error) { $doc->print($self->{OBJECT2}->errorMessage); }
  }
  else
  {
    $self->{OBJECT} = Portal::Accounting::Payroll::Objects::State->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode} );
    if ($self->{OBJECT}->error) { $doc->print($self->{OBJECT}->errorMessage); }
    $doc->print($self->{OBJECT}->printForm(form => 'selector'));
    if ($self->{OBJECT}->error) { $doc->print($self->{OBJECT}->errorMessage); }
  }
  return $doc;
}

sub c_save
{
  my $self = shift;
  if ($self->{input}->{states} ne '' && $self->{input}->{employee} ne '' )
  {
    eval("use Portal::Accounting::Payroll::Objects::State::$self->{input}->{states};");
    eval('$self->{OBJECT} = Portal::Accounting::Payroll::Objects::State::'.$self->{input}->{states}.'->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode}, employee => $self->{input}->{employee});');
    if ($self->{OBJECT}->error) { $self->error($self->{OBJECT}->errorMessage); }
    else { my $doc = $self->SUPER::c_save(@_);  $doc->setOnload(replace => 1, code => '');  return $doc; }
  }
  return $self->c_new(@_);
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

PC & Web Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::AppState(3)

=cut