package Portal::Accounting::Reports::Detailed;
use strict;
use Portal::Accounting::Reports::Main;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

use Portal::Accounting::Reports::Objects::Detailed;
require Exporter;

@ISA = qw(Portal::Accounting::Reports::Main Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.8 $';

my %commands = (
  display => "Transaction Reports",
  run_report => "Transaction Reports",
);

sub c_display
{
  my $self = shift;
  return $self->c_new(@_);
}

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  if (defined $self->input->{ledgerCode}) { $self->{ledgerCode} = $self->{input}->{ledgerCode}; }
  else { $self->error('no ledgerCode.  Reports do not yet support running consolidated Ledgers');  }
  
  # do validation
  if (!$self->Portal::Accounting::Reports::Detailed::isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.
  $self->{commands} = \%commands;

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut
sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
#   if ($self->numInvalid() > 0 ,, $self->numMissing() > 0)
#   {
#     $self->postfixError($self->genErrorString("all"));
#     return 0;
#   }
  return 1;
}

sub c_debug
{
  my $self = shift;
  my $doc = $self->prepDoc;
  $doc->print('<h2>DEBUG AND TESTING FOR CUSTOMIZE</h2>');
  foreach my $foo (keys %{$self->{ACCOUNTS}})
  {
    $doc->print($foo."->".$self->{ACCOUNTS}->{$foo}->get('name')."<br>");
  }
  $doc->print("<hr>");
  foreach my $foo (keys %{$self->{SUBACCOUNTS}})
  {
    $doc->print($foo."->".$self->{SUBACCOUNTS}->{$foo}->get('name')."<br>");
  }
  return $doc;
}

sub prepDoc
{
  my $self = shift;
  my $doc;
  if ($self->{input}->{command} eq 'run_report') { $doc = $self->SUPER::prepDoc(@_, heading => ''); }
  else { $doc = $self->SUPER::prepDoc(@_ ); }
  $doc->setJavascriptInclude(file => $self->{jsDir}."Reports/Detailed.js");
  return $doc;
}

sub c_edit
{
  my $self = shift;
  my $embed;
  if (defined $self->{input}->{embed}) { $embed = $self->{input}->{embed}; }
  $self->{OBJECT} = Portal::Accounting::Reports::Objects::Detailed->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode}, embed => $embed);
  if ($@) { $self->error($@); return undef;  }
  if ($self->{OBJECT}->error) { $self->error($self->{OBJECT}->errorMessage); return undef;  }
  my $doc = $self->SUPER::c_edit(@_);
  return $doc;
}

sub c_run_report
{
  my $self = shift;
  my $doc = $self->prepDoc;
  $self->{OBJECT} = Portal::Accounting::Reports::Objects::Detailed->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode},);
  if ($@) { $self->error($@); return undef;  }
  if ($self->{OBJECT}->error) { $self->error($self->{OBJECT}->errorMessage); return undef;  }
  my $foo = $self->{OBJECT};
  my $v = $foo->validateForm(input => $self->{input}, form => 'display');
  if (!$v)
  {
    $doc->print("<p>ValidateForm failed!</p>\n".$foo->{formObj}->genErrorString('all'));
    return $doc;
  }
  my $p = $foo->populate(%{$self->{input}});
  if (!$p) { $self->error("<p>Populate failed!</p>\n".$foo->errorMessage); $foo->resetError; return undef; }
  if (!$self->error && !$self->{OBJECT}->error)
  {
    my $format = $self->{OBJECT}->get('format');
    if ($format eq 'csv')
    {
      $doc = HTMLObject::Normal->new;
      $doc->setContentType('text/plain');
      my @plug = $self->{OBJECT}->selectSQL();
      if ($self->{OBJECT}->error) { $self->error($self->{OBJECT}->errorMessage); }
      else
      {
        my $sql = shift @plug;
        my $text = $self->dbHandle->readCSV(sql => $sql, plug => \@plug);
        if ($self->dbHandle->error) { $self->error($self->dbHandle->errorMessage); }
        else { $doc->print($text); }
      }
    }
    else
    {
      my $class = 'detail hyper';
      my $numRows = 1;
      if ($format eq 'print')
      {
        $class = 'detail print';
        $numRows = 0;
        $doc->print('<div style="text-align: center">'.$self->{OBJECT}->get('report_name').'</div>');
      }
      #output hidden version of the form so that user can resubmit
      my %form = $self->{OBJECT}->printForm(form => 'display');
      if ($self->{OBJECT}->error) { $self->error($self->{OBJECT}->errorMessage); }
      elsif ($format eq 'print')
      {
        my %form2 = (css => $form{css});
        $doc->print(%form2);
      }
      else
      {
        $doc->print(%form);
      }
      my $html = $self->{OBJECT}->printHTML(displayNumRows => $numRows, tableClass => $class, alternateRows => 0,);
      if ($self->{OBJECT}->error) { $doc->print($self->{OBJECT}->errorMessage); }
      else { $doc->print($html); }
  #     $doc->print("<div style=\"background-color: #abcdef\">".$self->{OBJECT}->{SELECT}[0]."</div>");
  #     if ($self->{OBJECT}->error) { $self->error($self->{OBJECT}->errorMessage); }
  #     $doc->print(join(',', @{$self->{OBJECT}->{PLUG} } ) );
    }
  }
  return $doc;
}

1;
__END__;